import { LfDataDataset } from "../framework/data.declarations";
import { LfThemeIcon, LfThemeInterface } from "../framework/theme.declarations";
import { LfMessengerChildTypes, LfMessengerImageRootIds, LfMessengerImageTypes, LfMessengerOptionTypes, LfMessengerUI } from "./messenger.declarations";
export declare const LF_MESSENGER_BLOCKS: {
    readonly character: {
        readonly _: "character";
        readonly avatar: "avatar";
        readonly biography: "biography";
        readonly image: "image";
        readonly label: "label";
        readonly name: "name";
        readonly nameWrapper: "name-wrapper";
        readonly status: "status";
    };
    readonly chat: {
        readonly _: "chat";
        readonly chat: "chat";
        readonly expander: "expander";
        readonly navigation: "navigation";
    };
    readonly covers: {
        readonly _: "covers";
        readonly add: "add";
        readonly images: "images";
        readonly label: "label";
        readonly title: "title";
    };
    readonly extraContext: {
        readonly _: "extra-context";
        readonly list: "list";
        readonly options: "options";
    };
    readonly form: {
        readonly _: "form";
        readonly button: "button";
        readonly confirm: "confirm";
        readonly field: "field";
        readonly label: "label";
    };
    readonly list: {
        readonly _: "list";
        readonly actions: "actions";
        readonly image: "image";
    };
    readonly messenger: {
        readonly _: "messenger";
    };
    readonly options: {
        readonly _: "options";
        readonly blocker: "blocker";
        readonly blockerIcon: "blocker-icon";
        readonly blockerLabel: "blocker-label";
        readonly cover: "cover";
        readonly info: "info";
        readonly label: "label";
        readonly name: "name";
        readonly placeholder: "placeholder";
        readonly placeholderIcon: "placeholder-icon";
        readonly wrapper: "wrapper";
    };
    readonly roster: {
        readonly _: "roster";
        readonly emptyData: "empty-data";
        readonly image: "image";
        readonly label: "label";
        readonly name: "name";
        readonly portrait: "portrait";
    };
};
/**
 * Returns a clean/default UI configuration object for the LF Messenger component.
 *
 * @returns {LfMessengerUI} An object containing the default UI state with:
 * - customizationView: View state for customization panel
 * - filters: States for different filter visibility options
 * - form: States for different form element visibility
 * - options: Available feature toggles
 * - panels: Collapse states for left and right panels
 */
export declare const LF_MESSENGER_CLEAN_UI: () => LfMessengerUI;
export declare const LF_MESSENGER_PARTS: {
    readonly emptyData: "empty-data";
    readonly messenger: "messenger";
    readonly roster: "roster";
};
export declare const LF_MESSENGER_PROPS: ["lfAutosave", "lfDataset", "lfStyle", "lfValue"];
/**
 * Object containing constant ID values used throughout the messenger component.
 * @constant
 * @property {Object} chat - IDs related to chat elements
 * @property {string} chat.leftExpander - ID for the left expansion control
 * @property {string} chat.rightExpander - ID for the right expansion control
 * @property {Object} options - IDs related to option buttons
 * @property {string} options.back - ID for the back navigation button
 * @property {string} options.customize - ID for the customization button
 */
export declare const LF_MESSENGER_IDS: {
    readonly chat: {
        readonly leftExpander: "left-expander";
        readonly rightExpander: "right-expaner";
    };
    readonly options: {
        readonly back: "back-button";
        readonly customize: "customize-button";
    };
};
export declare const AVATAR_COVER: LfThemeIcon;
export declare const LOCATION_COVER: LfThemeIcon;
export declare const OUTFIT_COVER: LfThemeIcon;
export declare const STYLE_COVER: LfThemeIcon;
export declare const TIMEFRAME_COVER: LfThemeIcon;
/**
 * Array containing various cover icon types used in the messenger component.
 * @constant {string[]} COVER_ICONS
 */
export declare const COVER_ICONS: readonly ["id", "sunset-2", "shirt", "color-swatch", "calendar-clock"];
/**
 * Dataset representing filter options for the messenger component.
 * Contains a list of selectable nodes, each representing a different filter category.
 * Each node includes:
 * - description: A text description of what the filter shows
 * - icon: The cover image/icon for the filter category
 * - id: Unique identifier for the filter
 * - value: Display name of the filter category
 *
 * @constant
 * @type {LfDataDataset}
 */
export declare const LF_MESSENGER_FILTER: LfDataDataset;
export declare const LF_MESSENGER_MENU: (theme: LfThemeInterface) => LfDataDataset;
/**
 * Maps child component types to their corresponding image asset directory names.
 * This mapping is used to determine where to find images for different messenger child components.
 *
 * @constant
 * @type {Record<LfMessengerChildTypes, LfMessengerImageTypes>}
 *
 * @example
 * // If childType is 'avatar', the corresponding image directory would be 'avatars'
 * const imageDir = CHILD_ROOT_MAP[childType];
 */
export declare const CHILD_ROOT_MAP: Record<LfMessengerChildTypes, LfMessengerImageTypes>;
/**
 * Array of valid option types for the messenger component.
 * Contains predefined categories used for organizing messenger options.
 * @type {LfMessengerOptionTypes[]}
 * @readonly
 */
export declare const OPTION_TYPE_IDS: LfMessengerOptionTypes[];
export declare const IMAGE_TYPE_IDS: LfMessengerImageRootIds<LfMessengerImageTypes>[];
export declare const LF_MESSENGER_NAV: (theme: LfThemeInterface) => LfDataDataset;
//# sourceMappingURL=messenger.constants.d.ts.map